% Copyright Pieter Abbeel
%
% 3.4 Laser Beam Sensor Model: Probability
%
% tic; p3_4_laser_beam_sensor_model_probability; toc
% Elapsed time is 170.301526 seconds.
%
% clear workspace, command window, close all images:
clear; clc; close all;


%1:

load simple_world

laser_model{1}.p_hit = 0.95;
laser_model{1}.p_short = 0.02;
laser_model{1}.p_max = 0.02;
laser_model{1}.p_rand = 0.01;
laser_model{1}.sigma_hit = 0.05;
laser_model{1}.lambda_short = 1;
laser_model{1}.z_max = 20;
laser_model{1}.z_max_eps = 0.02;
laser_model{1}.laser_angles = -pi: (pi/20) : pi;


s = [3.5; 3.5; pi];
z1 = [    0.4915    0.5090    0.5186    0.5724    0.6320    0.7073    0.6192    0.5535    0.5210    0.5028    0.4926    0.5214    0.5128    0.5477    0.6241    2.1110    1.8409    1.6633    1.5763    1.5252 ...
    1.4966    1.5174    1.5883   12.5293    2.5402    2.1187    0.6194    0.5646    0.5012    0.4964    0.4051    0.4966    0.5226    0.5577   18.2397    6.9200    0.6348    0.5430    0.5307    0.5027     0.5172]';

curr_weight = 1; % this is passed into measurement probability computations; this is useful for later: 
% if a particle has zero weight already (perhaps b/c you found it's in a
% occupied space), then the measurement function can decide to skip over it

p = beam_range_finder_measurement_probability_gridworld_batch(s, curr_weight, z1, laser_model{1}, gridworld); %YOURS to implement


%2:

load simple_world

% model 2: noise free 
laser_model{2}.p_hit = 1.0;
laser_model{2}.p_short = 0;
laser_model{2}.p_max = 0;
laser_model{2}.p_rand = 0;
laser_model{2}.sigma_hit = 0;
laser_model{2}.lambda_short = 1;
laser_model{2}.z_max = 20;
laser_model{2}.z_max_eps = 0.02;
laser_model{2}.laser_angles = -pi: (pi/20) : pi;

z2 = beam_range_finder_sampler_gridworld(s(1), s(2), s(3), gridworld, laser_model{2});

figure; draw_gridworld(gridworld); axis equal; hold on;
plot(s(1), s(2), 'x');
for k=1:length(z2)
	plot([s(1) s(1)+z2(k)*cos(s(3)+laser_model{2}.laser_angles(k))], [s(2) s(2)+z2(k)*sin(s(3)+laser_model{2}.laser_angles(k))],'r-');
end


sigmas_to_consider = [0.001 0.01 0.02 0.05 0.1 0.2 0.5 1 2];
delta_x_to_consider = -1.49:.01:0.49;
delta_y_to_consider = -0.49:.01:0.49;
delta_theta_to_consider = -pi:.01:pi;

%model 3: back to model 1, but we are varying sigma_hit:

% YOUR code here --- examine how log likelihood changes as a function of
% perturbing x, y, and theta 


%3:

% given the world, measurement, and laser model:

load rooms_world.mat
figure; draw_gridworld(gridworld); axis equal; hold on;

% same as laser model 1
laser_model{4}.p_hit = 0.95;
laser_model{4}.p_short = 0.02;
laser_model{4}.p_max = 0.02;
laser_model{4}.p_rand = 0.01;
laser_model{4}.sigma_hit = 0.05;
laser_model{4}.lambda_short = 1;
laser_model{4}.z_max = 20;
laser_model{4}.z_max_eps = 0.02;
laser_model{4}.laser_angles = -pi: (pi/20) : pi;

z3 = [6.8822   20.0000    3.8216    2.6264    1.7133    0.0562    1.4416    1.8022    4.3981    4.3138    4.1595    1.1808    1.2298    1.2879    1.4397 ...
    1.6240    2.0003    2.6448    3.8609    6.2471    6.1991    5.1672    2.6185    1.7647    1.3249    1.2574    1.0381    0.9318    0.8231    1.0530 ...
    3.8328    3.9047    4.0226    1.9069    1.4137    1.1003    1.3601    1.7627    2.6650    5.9936    6.8842]';


% plot for orientation theta = pi/2, as a function of x, y, what the log
% likelihood is of the pose being (x,y,theta) at a resolution of 0.1 in x
% and y

xs = 0:0.1:10;
ys = 0:0.1:15;

for i=1:length(xs)
	for j=1:length(ys)
		f(j,i) = log(beam_range_finder_measurement_probability_gridworld_batch([xs(i); ys(j); pi/2], curr_weight, z3, laser_model{4}, gridworld));
	end
end

[X,Y] = meshgrid(xs,ys);
figure; draw_gridworld(gridworld); axis equal; hold on;
contour(X,Y,f);


% for the same world and laser settings, I did
% s = [8.1; 7.3; pi/2];
% z = beam_range_finder_sampler_gridworld(s(1), s(2), s(3), gridworld,laser_model{4});
% % which gave me
% z =  [   3.2330    1.5716    0.6639    2.0487    1.6528    1.2541
% 1.2115    0.9296    0.9442    1.0180    0.9004    0.8898    0.9938
% 1.0413    1.1265    1.2464    1.5677    1.9160    1.8436    1.6866
% 1.6380    1.7393    1.7953    1.8759    1.8141    1.4562    1.4472
% 1.5573    4.2685    4.5291    7.1674    1.9098    1.1456    1.2414
% 1.2979    1.4731    1.8578    2.4159    3.5255    3.3513    3.2504]';
%
% I obtained laser_beam_contour_solution.fig; 
% 


% for the highest scoring x, y, refine the resolution in a 0.05
% neighborhood around it to 0.01 and recover the (x,y) with the highest
% log-likelihood:

% YOUR code here




% plot the measurements onto the map at the "max" location and let's see if
% it makes sense:


% YOUR code here



